import java.io.*;
import java.lang.*;
import java.math.BigInteger;
import java.util.Random;

public class RSAKeyGenerator
{
    public BigInteger n;                       // n = p x q
    public BigInteger d;                       // Private key d
    public BigInteger e;                       // Public key e

    // Euclid's algorithm: Computation of E from D and (p-1)(q-1)
    public BigInteger EUCLID(BigInteger D, BigInteger F) {
        BigInteger y;                   // y[i] = x[i-1] / x[i]
        BigInteger x2, x1, x0;          // x[i+1] = x[i-1] - y[i]*x[i]
        BigInteger a2, a1, a0;          // a[i+1] = a[i-1] - y[i]*a[i]
        BigInteger b2, b1, b0;          // b[i+1] = b[i-1] - y[i]*b[i]
        BigInteger E;

        x0 = F;                         // x[0] = (p-1)*(q-1) = F
        x1 = D;                         // x[1] = D
        a0 = BigInteger.valueOf(1);     // a[0] = 1
        a1 = BigInteger.valueOf(0);     // a[1] = 0
        b0 = BigInteger.valueOf(0);     // b[0] = 0
        b1 = BigInteger.valueOf(1);     // b[1] = 1
        E = BigInteger.valueOf(0);

        while (true) {
            y = x0.divide(x1);                  // y = x0 / x1
            x2 = x0.subtract(y.multiply(x1));   // x2 = x0 - y*x1
            a2 = a0.subtract(y.multiply(a1));   // a2 = a0 - y*a1
            b2 = b0.subtract(y.multiply(b1));   // b2 = b0 - y*b1

            x0 = x1;
            x1 = x2;
            a0 = a1;
            a1 = a2;
            b0 = b1;
            b1 = b2;

            // Stopping Criterion: x[i] = 1
            if (x1.equals(BigInteger.valueOf(1))) {
                if (b1.compareTo(BigInteger.valueOf(0))==1)         // b1 = +ve
                    E = b1;
                else if (b1.compareTo(BigInteger.valueOf(0))==-1)   // b1 = -ve
                    E = b1.add(F);
                return E;
            }
        }
    }

	void Generate()
	{
        BigInteger p;
        BigInteger q;
        BigInteger F;

	    Random RND = new Random();              // Random generator
	    do {
	        do {
                p = new BigInteger(28, 2, RND); // Generate a 28-bit probably prime number p
            } while (!p.isProbablePrime(2));    // Ensure p is a probably prime number
	        do {
                q = new BigInteger(28, 2, RND); // Generate a 28-bit probably prime number q
            } while (!q.isProbablePrime(2));    // Ensure q is a probably prime number

            n = p.multiply(q);                  // Calculate n(55/56-bit) = p(28-bit) x q(28-bit)
        } while (n.bitLength()!=56);            // Ensure n is a 56-bit number for 56-bit encryption

        // Calculate F = (p - 1) x (q - 1)
        F = (p.subtract(BigInteger.valueOf(1))).multiply((q.subtract(BigInteger.valueOf(1))));

        // Random select private key d s.t. [3 <= d <= F]
        do {
            int BRND = (java.lang.Math.abs(RND.nextInt()) % 55) + 2;
            d = new BigInteger(BRND, 2, RND);
        } while (d.compareTo(BigInteger.valueOf(3))==-1 || d.compareTo(F)==1);

        // Compute public key e using Euclid's algorithm
        e = EUCLID(d, F);
	}
} 